'use strict';

const AWS = require('aws-sdk');
const async = require('async');

const ec2 = new AWS.EC2();

const spawnInstance = (params, callback) => {
  AWS.config.update({
    region: params.region || 'us-west-1',
    accessKeyId: process.env.AWS_ACCESS_KEY_ID || params.accessKeyId,
    secretAccessKey: process.env.AWS_SECRET_ACCESS_KEY || params.secretAccessKey
  });

  if (params.region) {
    delete params.region;
  }

  if (params.accessKeyId) {
    delete params.accessKeyId;
  }

  if (params.secretAccessKey) {
    delete params.secretAccessKey;
  }

  params = Object.assign({
    ImageId: 'ami-96f1c1c4',
    InstanceType: 't2.micro',
    MinCount: 1,
    MaxCount: 2
  }, params);

  ec2.runInstances(params, (err, data) => {
    if (err) {
      return callback(err);
    }

    callback(null, data.Instances);
  });
};

const getInfo = (instances, callback) => {
  let params = {
    InstanceIds: []
  };

  instances.forEach(instance => {
    params.InstanceIds.push(instance.InstanceId);
  });

  ec2.describeInstances(params, (err, data) => {
    if (err) {
      return callback(err);
    }

    callback(null, data.Reservations[0].Instances);
  });
};

const createInstances = (params, callback) => {
  async.waterfall([
    async.apply(spawnInstance, params),
    getInfo
  ], (err, results) => {
    if (err) {
      return callback(err);
    }

    async.map(results, (instance, cb) => {
      let info = {
        'Instance ID': instance.InstanceId,
        'Private DNS': instance.PrivateDnsName,
        'Private IP': instance.PrivateIpAddress,
        'Public DNS': instance.PublicDnsName,
        'Public IP': instance.PublicIpAddress,
        Type: instance.InstanceType,
        Region: instance.Placement.AvailabilityZone
      };

      return cb(null, info);
    }, (mapErr, data) => {
      if (mapErr) {
        return callback(err);
      }

      callback(null, data);
    });
  });
};

module.exports = {
  create: createInstances
};
